@@ -1,8 +1,7 @@
-import inspect
+import datajoint as dj
 import pathlib
 import importlib
-
-import datajoint as dj
+import inspect
 from element_interface.utils import find_root_directory, find_full_path
 
 schema = dj.schema()
@@ -13,36 +12,32 @@ _linking_module = None
 def activate(
     scan_schema_name, *, create_schema=True, create_tables=True, linking_module=None
 ):
-    """Activate this schema.
-
-    Args:
-        scan_schema_name (str): schema name on the database server to activate the `scan` module
-        create_schema (bool): when True (default), create schema in the database if it does not yet exist.
-        create_tables (bool): when True (default), create tables in the database if they do not yet exist.
-        linking_module (str): a module name or a module containing the required dependencies to activate
-            the `scan` module.
-
-    Dependencies:
-    Upstream tables:
-        + Session: parent table to Scan, typically identifying a recording session
-        + Equipment: Reference table for Scan, specifying the equipment used for the acquisition of this scan
-        + Location: Reference table for ScanLocation, specifying the brain location where this scan is acquired
-
-    Functions:
-        + get_imaging_root_data_dir() -> list
-            Retrieve the full path for the root data directory - e.g. containing the imaging recording files or analysis results for all subject/sessions.
-            :return: a string (or list of string) for full path to the root data directory
-        + get_scan_image_files(scan_key: dict) -> list
-            Retrieve the list of ScanImage files associated with a given Scan
-            :param scan_key: key of a Scan
-            :return: list of ScanImage files' full file-paths
-        + get_scan_box_files(scan_key: dict) -> list
-            Retrieve the list of Scanbox files (*.sbx) associated with a given Scan
-            :param scan_key: key of a Scan
-            :return: list of Scanbox files' full file-paths
-        + get_processed_root_data_dir() -> str:
-            Retrieves the root directory for all processed data to be found from or written to
-            :return: a string for full path to the root directory for processed data
+    """
+    activate(scan_schema_name, *, create_schema=True, create_tables=True, linking_module=None)
+        :param scan_schema_name: schema name on the database server to activate the `scan` module
+        :param create_schema: when True (default), create schema in the database if it does not yet exist.
+        :param create_tables: when True (default), create tables in the database if they do not yet exist.
+        :param linking_module: a module name or a module containing the
+         required dependencies to activate the `scan` module:
+            Upstream tables:
+                + Session: parent table to Scan, typically identifying a recording session
+                + Equipment: Reference table for Scan, specifying the equipment used for the acquisition of this scan
+                + Location: Reference table for ScanLocation, specifying the brain location where this scan is acquired
+            Functions:
+                + get_imaging_root_data_dir() -> list
+                    Retrieve the full path for the root data directory - e.g. containing the imaging recording files or analysis results for all subject/sessions.
+                    :return: a string (or list of string) for full path to the root data directory
+                + get_scan_image_files(scan_key: dict) -> list
+                    Retrieve the list of ScanImage files associated with a given Scan
+                    :param scan_key: key of a Scan
+                    :return: list of ScanImage files' full file-paths
+                + get_scan_box_files(scan_key: dict) -> list
+                    Retrieve the list of Scanbox files (*.sbx) associated with a given Scan
+                    :param scan_key: key of a Scan
+                    :return: list of Scanbox files' full file-paths
+                + get_processed_root_data_dir() -> str:
+                    Retrieves the root directory for all processed data to be found from or written to
+                    :return: a string for full path to the root directory for processed data
     """
 
     if isinstance(linking_module, str):
@@ -66,16 +61,18 @@ def activate(
 
 
 def get_imaging_root_data_dir() -> str:
-    """Retrieve the root data directories containing the imaging data
-    for all subjects/sessions (e.g. acquired ScanImage raw files, output files from
-    processing routines, etc.). All data paths, directories in DataJoint Elements are
-    recommended to be stored as relative paths (posix format), with respect to some
-    user-configured "root" directory, which varies from machine to machine
-    (e.g. different mounted drive locations).
+    """
+    All data paths, directories in DataJoint Elements are recommended to be stored as
+    relative paths (posix format), with respect to some user-configured "root" directory,
+     which varies from machine to machine (e.g. different mounted drive locations)
 
-    Returns:
-        A string for full path to the imaging root data directory, or list of strings for
-        possible root data directories.
+    get_imaging_root_data_dir() -> list
+        This user-provided function retrieves the possible root data directories
+         containing the imaging data for all subjects/sessions
+         (e.g. acquired ScanImage raw files,
+         output files from processing routines, etc.)
+        :return: a string for full path to the imaging root data directory,
+         or list of strings for possible root data directories
     """
 
     root_directories = _linking_module.get_imaging_root_data_dir()
@@ -89,14 +86,10 @@ def get_imaging_root_data_dir() -> str:
 
 
 def get_processed_root_data_dir() -> str:
-    """Retrieve the root directory for all processed data. All data paths, directories
-    in DataJoint Elements are recommended to be stored as relative paths (posix format),
-    with respect to some user-configured "root" directory, which varies from machine to
-    machine (e.g. different mounted drive locations).
-
-
-    Returns:
-        A string for full path to the root directory for processed data.
+    """
+    get_processed_root_data_dir() -> str:
+        Retrieves the root directory for all processed data to be found from or written to
+        :return: a string for full path to the root directory for processed data
     """
 
     if hasattr(_linking_module, "get_processed_root_data_dir"):
@@ -106,49 +99,37 @@ def get_processed_root_data_dir() -> str:
 
 
 def get_scan_image_files(scan_key: dict) -> list:
-    """Retrieve the list of ScanImage files associated with a given Scan.
-
-    Args:
-        scan_key: Primary key of a Scan entry.
-
-    Returns:
-        A list of ScanImage files' full file-paths.
+    """
+    Retrieve the list of ScanImage files associated with a given Scan
+    :param scan_key: key of a Scan
+    :return: list of ScanImage files' full file-paths
     """
     return _linking_module.get_scan_image_files(scan_key)
 
 
 def get_scan_box_files(scan_key: dict) -> list:
-    """Retrieve the list of Scanbox files (*.sbx) associated with a given Scan.
-
-    Args:
-        scan_key: Primary key of a Scan entry.
-
-    Returns:
-        A list of Scanbox files' full file-paths.
+    """
+    Retrieve the list of Scanbox files (*.sbx) associated with a given Scan
+    :param scan_key: key of a Scan
+    :return: list of Scanbox files' full file-paths
     """
     return _linking_module.get_scan_box_files(scan_key)
 
 
 def get_nd2_files(scan_key: dict) -> list:
-    """Retrieve the list of Nikon files (*.nd2) associated with a given Scan.
-
-    Args:
-        scan_key: Primary key of a Scan entry.
-
-    Returns:
-        A list of Nikon files' full file-paths.
+    """
+    Retrieve the list of Nikon files (*.nd2) associated with a given Scan
+    :param scan_key: key of a Scan
+    :return: list of Nikon files' full file-paths
     """
     return _linking_module.get_nd2_files(scan_key)
 
 
 def get_prairieview_files(scan_key: dict) -> list:
-    """Retrieve the list of Bruker PrairieView tif files (*.tif) with a given Scan
-
-    Args:
-        scan_key: Primary key of a Scan entry.
-
-    Returns:
-        A list of Bruker PrairieView files' full file-paths.
+    """
+    Retrieve the list of Bruker PrairieView tif files (*.tif) with a given Scan
+    :param scan_key: key of a Scan
+    :return: list of Bruker PrairieView files' full file-paths
     """
     return _linking_module.get_prairieview_files(scan_key)
 
@@ -158,9 +139,7 @@ def get_prairieview_files(scan_key: dict) -> list:
 
 @schema
 class AcquisitionSoftware(dj.Lookup):
-    """A list of acquisition softwares supported by the workflow. Required to define a scan."""
-
-    definition = """  # Acquisition softwares
+    definition = """  # Name of acquisition software - e.g. ScanImage, Scanbox, NIS, PrairieView
     acq_software: varchar(24)    
     """
     contents = zip(["ScanImage", "Scanbox", "NIS", "PrairieView"])
@@ -176,10 +155,6 @@ class Channel(dj.Lookup):
 
 @schema
 class Scan(dj.Manual):
-    """A scan entry in this table is defined by a measurement done using a scanner (Equipment)
-    and an acquisition software. The details of the scanning data is placed in other tables,
-    including, ScanLocation, ScanInfo, and ScanInfo's part tables."""
-
     definition = """    
     -> Session
     scan_id: int        
@@ -192,25 +167,16 @@ class Scan(dj.Manual):
 
 @schema
 class ScanLocation(dj.Manual):
-    """Anatomical location of the scanned region in the brain
-
-    Refer to the `definition` attribute for the table design."""
-
-    definition = """Anatomical location
+    definition = """
     -> Scan   
-    ---
+    ---    
     -> Location      
     """
 
 
 @schema
 class ScanInfo(dj.Imported):
-    """
-    Information about the scan extracted from the recorded files.
-    Refer to the `definition` property to see the table design.
-    """
-
-    definition = """ # General data about the reso/meso scans from header
+    definition = """ # general data about the reso/meso scans from header
     -> Scan
     ---
     nfields              : tinyint   # number of fields
@@ -231,8 +197,6 @@ class ScanInfo(dj.Imported):
     """
 
     class Field(dj.Part):
-        """Stores field information of scan, including its coordinates, size, pixel pitch, etc."""
-
         definition = """ # field-specific scan information
         -> master
         field_idx         : int
@@ -249,11 +213,9 @@ class ScanInfo(dj.Imported):
         """
 
     class ScanFile(dj.Part):
-        """Filepath of the scan relative to root data directory"""
-
         definition = """
         -> master
-        file_path: varchar(255)  # Filepath relative to root data directory
+        file_path: varchar(255)  # filepath relative to root data directory
         """
 
     def make(self, key):
@@ -331,8 +293,10 @@ class ScanInfo(dj.Imported):
                             field_idx=plane_idx,
                             px_height=scan.image_height,
                             px_width=scan.image_width,
-                            um_height=getattr(scan, "image_height_in_microns", None),
-                            um_width=getattr(scan, "image_width_in_microns", None),
+                            um_height=getattr(
+                                scan, "image_height_in_microns", None),
+                            um_width=getattr(
+                                scan, "image_width_in_microns", None),
                             field_x=x_zero if x_zero else None,
                             field_y=y_zero if y_zero else None,
                             field_z=z_zero + scan.scanning_depths[plane_idx]
@@ -376,7 +340,8 @@ class ScanInfo(dj.Imported):
                     fps=sbx_meta["frame_rate"],
                     bidirectional=sbx_meta == "bidirectional",
                     nrois=sbx_meta["num_rois"] if is_multiROI else 0,
-                    scan_duration=(sbx_meta["num_frames"] / sbx_meta["frame_rate"]),
+                    scan_duration=(
+                        sbx_meta["num_frames"] / sbx_meta["frame_rate"]),
                 )
             )
             # Insert Field(s)
@@ -412,7 +377,8 @@ class ScanInfo(dj.Imported):
 
             # Frame per second
             try:
-                fps = 1000 / nd2_file.experiment[0].parameters.periods[0].periodDiff.avg
+                fps = 1000 / \
+                    nd2_file.experiment[0].parameters.periods[0].periodDiff.avg
             except:
                 fps = 1000 / nd2_file.experiment[0].parameters.periodDiff.avg
 
@@ -484,7 +450,8 @@ class ScanInfo(dj.Imported):
             from element_interface import prairieviewreader
 
             scan_filepaths = get_prairieview_files(key)
-            pvscan_info = prairieviewreader.get_pv_metadata(scan_filepaths[0])
+            pvscan_info = prairieviewreader.get_pv_metadata(
+                scan_filepaths[0])
             self.insert1(
                 dict(
                     key,
@@ -526,7 +493,8 @@ class ScanInfo(dj.Imported):
             )
 
         # Insert file(s)
-        root_dir = find_root_directory(get_imaging_root_data_dir(), scan_filepaths[0])
+        root_dir = find_root_directory(
+            get_imaging_root_data_dir(), scan_filepaths[0])
 
         scan_files = [
             pathlib.Path(f).relative_to(root_dir).as_posix() for f in scan_filepaths
